using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CommunicatorAPI;
using System.Threading;
using System.Collections;

namespace Communicator
{
    public partial class Form1 : Form
    {
        CommunicatorAPI.Messenger communicator = null;
        bool connected = false;
        ArrayList watchList = new ArrayList();
        bool logAllUsers = false;
        bool alertWatchedUsers = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SignIn(null, null);
        }

        private void SignIn(string account, string password)
        {
            if (connected)
                return;
            if (communicator == null)
            {
                communicator = new CommunicatorAPI.Messenger();

                // Register event handlers for OnSignin and Signout events
                communicator.OnSignin += new DMessengerEvents_OnSigninEventHandler(communicator_OnSignin);
                communicator.OnSignout += new DMessengerEvents_OnSignoutEventHandler(communicator_OnSignout);
                communicator.OnContactStatusChange += new DMessengerEvents_OnContactStatusChangeEventHandler(OnContactStatusChange);
            }

            if (account == null || "".Equals(account))
                communicator.AutoSignin();
            else
                communicator.Signin(0, account, password);
        }

        void communicator_OnSignin(int hr)
        {
            if (hr != 0)
            {
                MessageBox.Show("SignIn Failed.");
                return;
            }
            if (connected)
                return;

            connected = true;
            populateContacts();          

            button1.Visible = false;
        }

        void OnContactStatusChange(object pMContact, MISTATUS mStatus)
        {
            IMessengerContact contact = pMContact as IMessengerContact;
            string status = Enum.GetName(typeof(MISTATUS), mStatus);
            Contact thisContact = new Contact(contact.FriendlyName, contact.SigninName, status);
            Thread t = new Thread(new ParameterizedThreadStart(showMessage));
            t.Start(thisContact);
        }

        private void populateContacts()
        {
            listBox1.Items.Clear();
            foreach (IMessengerContact contact in communicator.MyContacts as IMessengerContacts)
            {
                if (!contact.IsSelf)
                {
                    Contact thisContact = new Contact(contact.FriendlyName, contact.SigninName, Enum.GetName(typeof(MISTATUS), contact.Status));
                    listBox1.Items.Add(thisContact);
                }
            }
        }

        private void refreshContacts()
        {
            listBox1.Items.Clear();
            listBox2.Items.Clear();
            foreach (IMessengerContact contact in communicator.MyContacts as IMessengerContacts)
            {
                if (!contact.IsSelf)
                {
                    Contact thisContact = new Contact(contact.FriendlyName, contact.SigninName, Enum.GetName(typeof(MISTATUS), contact.Status));
                    if (watchList.Contains(thisContact))
                        listBox2.Items.Add(thisContact);
                    else
                        listBox1.Items.Add(thisContact);
                }
            }
        }

        private void showMessage(object message)
        {
            Contact contact = (Contact)message;
            bool refresh = false;
            if (logAllUsers || watchList.Contains(contact))
            {
                listBox3.Items.Insert(0, message.ToString() + " - " + DateTime.Now);
                refresh = true;
            }
            if (alertWatchedUsers && watchList.Contains(contact))
            {
                MessageBox.Show(message.ToString());
                refresh = true;
            }
            if(refresh)
                refreshContacts();
        }

        void Signout()
        {
            if (!connected || communicator == null)
                return;
            communicator.Signout();
        }

        // Event handler for OnSignout event
        void communicator_OnSignout()
        {
            connected = false;
            // Release the unmanaged resource.
            System.Runtime.InteropServices.Marshal.ReleaseComObject(communicator);
            communicator = null;
        }

        private void notifyIcon1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            this.Visible = !this.Visible;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            object[] items = new object[listBox1.SelectedItems.Count + 100];
            listBox1.SelectedItems.CopyTo(items, 0);
            foreach (object i in items)
            {
                if (i == null)
                    break;
                listBox1.Items.Remove(i);
                listBox2.Items.Add(i);
                watchList.Add(i);
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            object[] items = new object[listBox2.SelectedItems.Count + 100];
            listBox2.SelectedItems.CopyTo(items, 0);
            foreach (object i in items)
            {
                if (i == null)
                    break;
                listBox2.Items.Remove(i);
                listBox1.Items.Add(i);
                watchList.Remove(i);
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            object[] items = new object[listBox3.Items.Count + 100];
            listBox3.Items.CopyTo(items, 0);
            StringBuilder sb = new StringBuilder();
            foreach (object i in items)
            {
                if (i == null)
                    break;
                sb.Append(i.ToString() + "\n");
            }
            Clipboard.SetText(sb.ToString());
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            logAllUsers = checkBox1.Checked;
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            alertWatchedUsers = checkBox2.Checked;
        }

        private void button5_Click(object sender, EventArgs e)
        {
            listBox3.Items.Clear();
        }
    }
}